import { fileIsExist } from '@kevisual/use-config';
import path from 'path';
import fs from 'fs';
import { appsPath } from '../lib/index.ts';

export const installAppFromKey = async (key: string) => {
  const directory = path.join(appsPath, key);
  if (!fileIsExist(directory)) {
    throw new Error('App not found');
  }
  const pkgs = path.join(directory, 'package.json');
  if (!fileIsExist(pkgs)) {
    throw new Error('Invalid package.json');
  }
  const json = fs.readFileSync(pkgs, 'utf-8');
  const pkg = JSON.parse(json);
  const { name, version, app } = pkg;
  if (!name || !version || !app) {
    throw new Error('Invalid package.json');
  }
  const readmeFile = path.join(directory, 'README.md');
  let readmeDesc = '';
  if (fileIsExist(readmeFile)) {
    readmeDesc = fs.readFileSync(readmeFile, 'utf-8');
  }
  let showAppInfo = {
    key,
    status: 'inactive',
    type: app?.type || 'system-app',
    description: readmeDesc || '',
    version,
    //
    entry: app?.entry || '',
    path: directory,
    origin: app,
  };
  app.key = key;
  fs.writeFileSync(pkgs, JSON.stringify(pkg, null, 2));
  return { pkg, showAppInfo };
};
export const getAppPathKeys = async () => {
  let files = fs.readdirSync(appsPath);
  files = files.filter((file) => {
    const stat = fs.statSync(path.join(appsPath, file));
    if (file === 'node_modules') return false;
    return stat.isDirectory();
  });
  return files;
};
