import { app } from '@/app.ts';
import { MicroAppUploadModel } from './models.ts';

// 获取MicroAppUpload的uploadList的接口
app
  .route({
    path: 'micro-app-upload',
    key: 'list',
    middleware: ['auth'],
    description: 'Get micro app upload list',
  })
  .define(async (ctx) => {
    const { uid } = ctx.state.tokenUser;
    const uploadList = await MicroAppUploadModel.findAll({
      // where: { uid },
    });
    ctx.body = uploadList;
  })
  .addTo(app);

// 获取单个MicroAppUpload的接口
app
  .route({
    path: 'micro-app-upload',
    key: 'get',
    middleware: ['auth'],
    description: 'Get a single micro app upload',
  })
  .define(async (ctx) => {
    const { id, title } = ctx.query;
    let upload: MicroAppUploadModel | null = null;
    if (id) {
      upload = await MicroAppUploadModel.findByPk(id);
    } else if (title) {
      upload = await MicroAppUploadModel.findOne({
        where: { title },
      });
    }
    if (upload) {
      ctx.body = upload;
    } else {
      ctx.throw(404, 'Not found');
    }
  })
  .addTo(app);

// 删除MicroAppUpload的接口
app
  .route({
    path: 'micro-app-upload',
    key: 'delete',
    middleware: ['auth'],
    description: 'Delete a micro app upload',
  })
  .define(async (ctx) => {
    const { id } = ctx.query;
    const deleted = await MicroAppUploadModel.destroy({
      where: { id },
    });
    ctx.body = { deleted };
  })
  .addTo(app);
